@@ -1,10 +1,10 @@
 from .molgrid import MolGrid
+from .callbacks import make_popup_callback
 from .dispatch import (display,
                        save)
 from .select import (get_selection,
                      list_grids)
-from .utils import (sdf_to_dataframe,
-                    make_popup_callback)
+from .utils import sdf_to_dataframe
 from ._version import __version__
 from .widget import _jupyter_labextension_paths, _jupyter_nbextension_paths
 
